﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Bulldozer.DrawningObjects;
using Bulldozer.Generics;
using Bulldozer.MovementStrategy;
using Bulldozer.Drawnings;
using Bulldozer.Exceptions;

namespace Bulldozer.Generics
{
    internal class BulldozersGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, BulldozerGenericCollection<DrawningBulldozer,
        DrawningObjectBulldozer>> _tractorStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _tractorStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public BulldozersGenericStorage(int pictureWidth, int pictureHeight)
        {
            _tractorStorages = new Dictionary<string,
            BulldozerGenericCollection<DrawningBulldozer, DrawningObjectBulldozer>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            // TODO Прописать логику для добавления
            if (!_tractorStorages.ContainsKey(name))
            {
                var tractorCollection = new BulldozerGenericCollection<DrawningBulldozer, DrawningObjectBulldozer>(_pictureWidth, _pictureHeight);
                _tractorStorages.Add(name, tractorCollection);
            }
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            // TODO Прописать логику для удаления
            if (_tractorStorages.ContainsKey(name))
            {
                _tractorStorages.Remove(name);
            }
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public BulldozerGenericCollection<DrawningBulldozer, DrawningObjectBulldozer>? this[string ind]
        {
            get
            {
                // TODO Продумать логику получения набора
                if (_tractorStorages.ContainsKey(ind))
                {
                    return _tractorStorages[ind];
                }
                return null;
            }
        }
        /// <summary>
        /// Сохранение информации по установкам в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string, BulldozerGenericCollection<DrawningBulldozer, DrawningObjectBulldozer>> record in _tractorStorages)
            {
                StringBuilder records = new();
                foreach (DrawningBulldozer? elem in record.Value.GetBulldozer)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                throw new ArgumentException("Невалидная операция, нет данных для сохранения");
            }
            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.Write($"BulldozerStorage{Environment.NewLine}{data}");
            }
        }
        /// <summary>
        /// Загрузка информации по установкам в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            using (StreamReader reader = new StreamReader(filename))
            {
                string cheker = reader.ReadLine();
                if (cheker == null)
                {
                    throw new ArgumentException("Нет данных для загрузки");
                }
                if (!cheker.StartsWith("BulldozerStorage"))
                {
                    throw new InvalidDataException("Неверный формат ввода");
                }
                _tractorStorages.Clear();
                string strs;
                bool firstinit = true;
                while ((strs = reader.ReadLine()) != null)
                {
                    if (strs == null && firstinit)
                    {
                        throw new ArgumentException("Нет данных для загрузки");
                    }
                    if (strs == null)
                    {
                        break;
                    }
                    firstinit = false;
                    string name = strs.Split(_separatorForKeyValue)[0];
                    BulldozerGenericCollection<DrawningBulldozer, DrawningObjectBulldozer> collection = new(_pictureWidth, _pictureHeight);
                    foreach (string data in strs.Split(_separatorForKeyValue)[1].Split(_separatorRecords))
                    {
                        DrawningBulldozer? bulldozer =
                        data?.CreateDrawningBulldozer(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (bulldozer != null)
                        {
                            try { _ = collection + bulldozer; }
                            catch (BulldozerNotFoundException e)
                            {
                                throw e;
                            }
                            catch (StorageOverflowException e)
                            {
                                throw e;
                            }
                        }
                    }
                    _tractorStorages.Add(name, collection);
                }
            }
        }
    }
}
